/**
 * Localization strings for the UI Multiselect widget
 *
 * @locale es, es-ES
 */
 
$.extend($.ui.multiselect.locale, {
	addAll:'Agregar todos',
	removeAll:'Remover todos',
	itemsCount:'#{count} Objetos seleccionados',
	itemsTotal:'#{count} Objetos total',
	busy:'por favor, espere...',
	//errorDataFormat:"Cannot add options, unknown data format",
	errorInsertNode:"Ha habido un problema al tratar de agregar el tema:\n\n\t[#{key}] => #{value}\n\nLa operación fue abortada.",
	errorReadonly:"La opción #{option} es de sólo lectura."//,
	//errorRequest:"Sorry! There seemed to be a problem with the remote call. (Type: #{status})"
});
